@@ -24,37 +24,6 @@ using namespace boost::program_options;
 
 #include "datatypes.h"
 
-//=== Xception =============================================================
-Xception::Xception (const string& message)
-	: m_message(message)
-{
-}
-
-Xception::Xception (const format& message)
-	: m_message(message.str())
-{
-}
-
-Xception::~Xception ()
-{
-}
-
-const string& Xception::Message () const
-{
-	return m_message;
-}
-
-//=== Xarning ==============================================================
-Xarning::Xarning (const string& message)
-	: Xception(message)
-{
-}
-
-Xarning::Xarning (const format& message)
-	: Xception(message)
-{
-}
-
 //=== Ratio ================================================================
 Ratio::Ratio (double numerator, double denominator)
 	: m_numerator(numerator)
