@@ -26,7 +26,8 @@ using namespace boost::gregorian;
 using namespace boost::program_options;
 using namespace Poco::Data;
 
-#include "datatypes.h"
+#include "xception.h"
+#include "conversions.h"
 #include "config.h"
 #include "poco_db.h"
 
@@ -45,7 +46,7 @@ PocoDatabase::PocoDatabase (const string& connect, bool verbose, const string& K
 PocoDatabase::~PocoDatabase ()
 {
 	if (m_verbose)
-		cout << "disconnecting from database" << endl;
+		encoded::cout << "disconnecting from database" << endl;
 
 	if (m_pSession && m_pSession->isConnected())
 		m_pSession->close();
@@ -60,25 +61,17 @@ void PocoDatabase::Connect () throw (Xception)
 	try
 	{
 		if (m_verbose)
-			cout << "connecting to database" << endl;
+			encoded::cout << "connecting to database" << endl;
 
 		m_pSession = new Session(m_KEY, m_connect);
 	}
-	catch (const DataException& x)
-	{
-		if (m_pSession)
-		{
-			delete m_pSession;
-			m_pSession = NULL;
-		}
-		throw Xception(format("cannot open database: %s, %s") % m_connect % x.displayText());
-	}
+	CATCH_CLEANUP_RETHROW(if (m_pSession) {delete m_pSession; m_pSession = NULL;}, format("cannot open database: %s") % m_connect);
 
 	// check schema
 	try
 	{
 		if (m_verbose)
-			cout << "checking schema" << endl;
+			encoded::cout << "checking schema" << endl;
 
 		(*m_pSession) << "select * from TCollections;", now;
 	}
@@ -134,10 +127,7 @@ void PocoDatabase::Connect () throw (Xception)
 			} while (rs.moveNext());
 		}
 	}
-	catch (...)
-	{
-		throw Xception("error getting schema meta data informations");
-	}
+	CATCH_RETHROW("error getting schema meta data informations")
 }
 
 const PocoDatabase::CollectionMetaInfo& PocoDatabase::GetCollectionMetaInfo (const string& name) const throw (Xception)
@@ -145,7 +135,7 @@ const PocoDatabase::CollectionMetaInfo& PocoDatabase::GetCollectionMetaInfo (con
 	CollectionMetaInfo cmi = {name, '\0', "", ""};
 	CollectionMetaInfos::const_iterator I = m_collectionMetaInfos.find(cmi);
 	if (I == m_collectionMetaInfos.end())
-		throw Xception("unknown collection name");
+		THROW(format("unknown collection name: %s") % name);
 	return *I;
 }
 
@@ -168,9 +158,13 @@ DBTransactor::~DBTransactor ()
 
 void DBTransactor::Commit ()
 {
-	if (m_counter == 1)
+	try
 	{
-		m_pSession->commit();
-		m_pSession = NULL;
+		if (m_counter == 1)
+		{
+			m_pSession->commit();
+			m_pSession = NULL;
+		}
 	}
+	CATCH_RETHROW("could not commit transaction");
 }
