@@ -27,6 +27,8 @@ using namespace boost::program_options;
 using namespace Poco::Data;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "../libpdrx/config.h"
 #include "db_impl.h"
 
@@ -47,16 +49,20 @@ void PocoDatabaseImpl::Connect () throw (Xception)
 
 string PocoDatabaseImpl::GetNewTblname () const
 {
-	set<string> tblnames;
-	foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
+	try
 	{
-		tblnames.insert(cmi.m_tblname);
+		set<string> tblnames;
+		foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
+		{
+			tblnames.insert(cmi.m_tblname);
+		}
+		set<string>::const_iterator I = tblnames.end();
+		string s(*(--I));
+		s.erase(0, 1); // C
+		int i = lexical_cast<int>(s);
+		return (format("C%d") % ++i).str();
 	}
-	set<string>::const_iterator I = tblnames.end();
-	string s(*(--I));
-	s.erase(0, 1); // C
-	int i = lexical_cast<int>(s);
-	return (format("C%d") % ++i).str();
+	CATCH_RETHROW("could not get new table name")
 }
 
 char PocoDatabaseImpl::GetCollectionTypeFromValue (const any& a) const
@@ -70,13 +76,13 @@ char PocoDatabaseImpl::GetCollectionTypeFromValue (const any& a) const
 	if (a.type() == typeid(string))
 		return 't';
 
-	throw Xception("invalid type");
+	THROW("invalid collection type");
 }
 
 void PocoDatabaseImpl::ListCollections () throw (Xception)
 {
 	if (m_verbose)
-		cout << "listing collections" << endl;
+		encoded::cout << "listing collections" << endl;
 
 	struct fill {
 		const string& m_s;
@@ -88,45 +94,44 @@ void PocoDatabaseImpl::ListCollections () throw (Xception)
 		{
 			static const size_t widths[] = {6, 8, 6, 7, 20, 20, 16}; // real column widths
 			size_t width = widths[idx];
-			string result(m_s);
+			wstring result(lexical_cast<wstring>(m_s));
 			while (result.length() < width)
-				result += ' ';
+				result += L' ';
 			if (result.length() > width)
 				result.erase(width - 1);
-			return result + ' '; // + 1 space as column delimiter
+			result += L' '; // + 1 space as column delimiter
+			return lexical_cast<string>(result);
 		}
 	};
 
 	try
 	{
-		cout << "  " << fill("name")(0) << fill("type")(1) << fill("table")(2) << fill("recs")(3) << fill("first")(4) << fill("last")(5) << fill("unit")(6) << "purpose" << endl;
+		encoded::cout << "  " << fill("name")(0) << fill("type")(1) << fill("table")(2) << fill("recs")(3) << fill("first")(4) << fill("last")(5) << fill("unit")(6) << "purpose" << endl;
 
 		foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
 		{
-			cout << "  " << fill(cmi.m_collection)(0);
+			encoded::cout << "  " << fill(cmi.m_collection)(0);
 
 			switch (cmi.m_type)
 			{
-				case 'n':	cout << fill("numeric")(1); break;
-				case 'r':	cout << fill("ratio")(1); break;
-				case 't':	cout << fill("text")(1); break;
+				case 'n':	encoded::cout << fill("numeric")(1); break;
+				case 'r':	encoded::cout << fill("ratio")(1); break;
+				case 't':	encoded::cout << fill("text")(1); break;
 				default:	break;
 			}
 
-			cout << fill(cmi.m_tblname)(2);
+			encoded::cout << fill(cmi.m_tblname)(2);
 
 			int count;
 			string min, max;
 			(*m_pSession) << "select count(*),min(t),max(t) from " << cmi.m_tblname << ";", into(count), into(min), into(max), now;
-			cout << fill(lexical_cast<string>(count))(3) << fill(min)(4) << fill(max)(5);
+			encoded::cout << fill(lexical_cast<string>(count))(3) << fill(min)(4) << fill(max)(5);
 
-			cout << fill(cmi.m_unit)(6);
-			cout << cmi.m_purpose << endl;
+			encoded::cout << fill(cmi.m_unit)(6);
+			encoded::cout << cmi.m_purpose << endl;
 		}
 	}
-	catch (...)
-	{
-	}
+	CATCH_RETHROW("could not list collections")
 }
 
 void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "name[, n|r|t[, unit[, purpose]]]"
@@ -139,7 +144,7 @@ void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "n
 			regex rx("\\s*([^,\\s]+)(?:\\s*,\\s*([nrt])\\s*(?:,([^,]+)(?:,(.+))?)?)?");
 			smatch mr;
 			if (!regex_match(name, mr, rx))
-				throw Xception(format("illegal collection specification: %s") % name);
+				THROW(format("illegal collection specification: %s") % name);
 			n = mr[1];
 			t = mr[2];
 			if (t.empty())
@@ -164,7 +169,7 @@ void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "n
 				return;
 			}
 			else
-				throw Xception(format("collection %s already exists but differs in type: %c <-> %s") % n % cmi.m_type % t);
+				THROW(format("collection %s already exists but differs in type: %c <-> %s") % n % cmi.m_type % t);
 		}
 		catch (...)
 		{
@@ -173,10 +178,10 @@ void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "n
 
 		if (m_verbose)
 		{
-			cout << "adding collection " << n;
+			encoded::cout << "adding collection " << n;
 			if (!p.empty())
-				cout << " (" << p << ')';
-			cout << endl;
+				encoded::cout << " (" << p << ')';
+			encoded::cout << endl;
 		}
 
 		// build new table name
@@ -189,22 +194,19 @@ void PocoDatabaseImpl::AddCollection (const string& name) throw (Xception) // "n
 		CollectionMetaInfo cmi = {n, t[0], tblname, p, u};
 		m_collectionMetaInfos.insert(cmi);
 	}
-	catch (...)
-	{
-		throw Xception(format("could not create collection: %s") % name);
-	}
+	CATCH_RETHROW(format("could not create collection: %s") % name)
 }
 
 void PocoDatabaseImpl::DeleteCollection (const string& name) throw (Xception)
 {
 	if (m_verbose)
-		cout << "deleting collection" << endl;
+		encoded::cout << "deleting collection" << endl;
 
 	try
 	{
 		// check for built-in collection
 		if (name == "*" || name == "#")
-			throw Xception(format("cannot delete built-in collection: %s") % name);
+			THROW(format("cannot delete built-in collection: %s") % name);
 
 		// check for unknown collection
 		const CollectionMetaInfo& cmi = GetCollectionMetaInfo(name);
@@ -215,16 +217,13 @@ void PocoDatabaseImpl::DeleteCollection (const string& name) throw (Xception)
 		// forget the CollectionMetaInfo
 		m_collectionMetaInfos.erase(cmi);
 	}
-	catch (...)
-	{
-		throw Xception(format("could not delete collection: %s") % name);
-	}
+	CATCH_RETHROW(format("could not delete collection: %s") % name)
 }
 
 void PocoDatabaseImpl::DeleteAllCollections () throw (Xception)
 {
 	if (m_verbose)
-		cout << "deleting all collections" << endl;
+		encoded::cout << "deleting all collections" << endl;
 
 	try
 	{
@@ -249,16 +248,13 @@ void PocoDatabaseImpl::DeleteAllCollections () throw (Xception)
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not delete collections");
-	}
+	CATCH_RETHROW("could not delete collections")
 }
 
 void PocoDatabaseImpl::ListRejections () throw (Xception)
 {
 	if (m_verbose)
-		cout << "listing rejections" << endl;
+		encoded::cout << "listing rejections" << endl;
 
 	struct fill {
 		const string& m_s;
@@ -270,12 +266,13 @@ void PocoDatabaseImpl::ListRejections () throw (Xception)
 		{
 			static const size_t widths[] = {20}; // real column widths
 			size_t width = widths[idx];
-			string result(m_s);
+			wstring result(lexical_cast<wstring>(m_s));
 			while (result.length() < width)
-				result += ' ';
+				result += L' ';
 			if (result.length() > width)
 				result.erase(width - 1);
-			return result + ' '; // + 1 space as column delimiter
+			result += L' '; // + 1 space as column delimiter
+			return lexical_cast<string>(result);
 		}
 	};
 
@@ -288,56 +285,45 @@ void PocoDatabaseImpl::ListRejections () throw (Xception)
 		RecordSet rs(select);
 		if (rs.moveFirst())
 		{
-			cout << "  " << fill("timestamp")(0) << "expression" << endl;
+			encoded::cout << "  " << fill("timestamp")(0) << "expression" << endl;
 			do {
-				cout << "  " << fill(rs.value(0).convert<string>())(0) << rs.value(1).convert<string>() << endl;
+				encoded::cout << "  " << fill(rs.value(0).convert<string>())(0) << rs.value(1).convert<string>() << endl;
 			} while (rs.moveNext());
 		}
 	}
-	catch (...)
-	{
-	}
+	CATCH_RETHROW("could not list rejections")
 }
 
 void PocoDatabaseImpl::AddRejected (const ptime& timestamp, const string& expr) throw (Xception)
 {
 	try
 	{
-		const string& t = lexical_cast<string, ptime>(timestamp);
+		const string& t = lexical_cast<string>(timestamp);
 		(*m_pSession) << "insert into TRejected values (null,'" << t << "','" << expr << "');", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not insert rejection");
-	}
+	CATCH_RETHROW("could not insert rejection")
 }
 
 void PocoDatabaseImpl::DeleteRejected (const ptime& timestamp) throw (Xception)
 {
 	try
 	{
-		const string& t = lexical_cast<string, ptime>(timestamp);
+		const string& t = lexical_cast<string>(timestamp);
 		(*m_pSession) << "delete from TRejected where t='" << t << "';", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not delete rejection");
-	}
+	CATCH_RETHROW("could not delete rejection")
 }
 
 void PocoDatabaseImpl::DeleteAllRejections () throw (Xception)
 {
 	if (m_verbose)
-		cout << "deleting all rejections" << endl;
+		encoded::cout << "deleting all rejections" << endl;
 
 	try
 	{
 		(*m_pSession) << "delete from TRejected;", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not delete rejections");
-	}
+	CATCH_RETHROW("could not delete rejections")
 }
 
 char PocoDatabaseImpl::GetCollectionType (const string& name) const throw (Xception)
@@ -395,10 +381,7 @@ void PocoDatabaseImpl::GetCollectionItems (const string& name, CollectionItems&
 			} while (rs.moveNext());
 		}
 	}
-	catch (...)
-	{
-		throw Xception("could not get collection data");
-	}
+	CATCH_RETHROW("could not select collection data")
 }
 
 void PocoDatabaseImpl::AddCollectionsItems (const CollectionsItems& items) throw (Xception)
@@ -414,10 +397,7 @@ void PocoDatabaseImpl::AddCollectionsItems (const CollectionsItems& items) throw
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not add collection data");
-	}
+	CATCH_RETHROW("could not add collection data")
 }
 
 void PocoDatabaseImpl::DeleteCollectionsItems (const ptime& timestamp, const Collections& collections)
@@ -435,10 +415,29 @@ void PocoDatabaseImpl::DeleteCollectionsItems (const ptime& timestamp, const Col
 
 		transactor.Commit();
 	}
-	catch (...)
+	CATCH_RETHROW("could not delete collection data")
+}
+
+ptime PocoDatabaseImpl::GetYoungestCollectionItemAtAll () const throw (Xception)
+{
+	try
 	{
-		throw Xception("could not delete collection data");
+		ptime timestamp;
+		foreach (const CollectionMetaInfo& cmi, m_collectionMetaInfos)
+		{
+			string max;
+			(*m_pSession) << "select max(t) from " << cmi.m_tblname << ";", into(max), now;
+
+			if (!max.empty())
+			{
+				ptime t = lexical_cast<ptime>(max);
+				if (timestamp == not_a_date_time || timestamp < t)
+					timestamp = t;
+			}
+		}
+		return timestamp;
 	}
+	CATCH_RETHROW("could not select collection data")
 }
 
 void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptime& timestamp, const string& rejected) const throw (Xception)
@@ -558,8 +557,5 @@ void PocoDatabaseImpl::GenerateExpressions (Expressions& expressions, const ptim
 			}
 		}
 	}
-	catch (...)
-	{
-		throw Xception("error selecting data");
-	}
+	CATCH_RETHROW("could not generate expressions from collection data")
 }
