@@ -14,10 +14,10 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#ifdef USE_MYSQL
-
 #include "../libpdrx/common.h"
 
+#ifdef USE_MYSQL
+
 #include <Poco/Data/Common.h>
 #include <Poco/Data/MySQL/Connector.h>
 #include <Poco/Data/MySQL/MySQLException.h>
@@ -31,6 +31,8 @@ using namespace boost::program_options;
 using namespace Poco::Data;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "../libpdrx/config.h"
 #include "db_impl.h"
 
@@ -50,7 +52,7 @@ MySQLDatabase::MySQLDatabase (const string& connect, bool verbose)
 void MySQLDatabase::NoSchemaCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "creating new schema" << endl;
+		encoded::cout << "creating new schema" << endl;
 
 	try
 	{
@@ -61,7 +63,7 @@ void MySQLDatabase::NoSchemaCallback () throw (Xception)
 					"name varchar(64) not null primary key,"
 					"type char(1) not null,"
 					"tbl_name varchar(64) not null,"
-					"purpose varchar(256)",
+					"purpose varchar(256),"
 					"unit varchar(16)"
 				 ") engine=InnoDB;", now;
 
@@ -89,40 +91,31 @@ void MySQLDatabase::NoSchemaCallback () throw (Xception)
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not create schema");
-	}
+	CATCH_RETHROW("could not create schema")
 }
 
 void MySQLDatabase::NoPurposeColumnCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "updating schema" << endl;
+		encoded::cout << "updating schema" << endl;
 
 	try
 	{
 		(*m_pSession) << "alter table TCollections add column purpose varchar(256);", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update schema");
-	}
+	CATCH_RETHROW("could not update schema")
 }
 
 void MySQLDatabase::NoUnitColumnCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "updating schema" << endl;
+		encoded::cout << "updating schema" << endl;
 
 	try
 	{
 		(*m_pSession) << "alter table TCollections add column unit varchar(16);", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update schema");
-	}
+	CATCH_RETHROW("could not update schema")
 }
 
 void MySQLDatabase::CreateCollectionInSchema (const string& name, char type, const string& tblname, const string& purpose, const string& unit) throw (Xception)
@@ -149,10 +142,7 @@ void MySQLDatabase::CreateCollectionInSchema (const string& name, char type, con
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not create collection");
-	}
+	CATCH_RETHROW("could not create collection table")
 }
 
 void MySQLDatabase::DropCollectionFromSchema (const string& tblname) throw (Xception)
@@ -166,10 +156,7 @@ void MySQLDatabase::DropCollectionFromSchema (const string& tblname) throw (Xcep
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not delete collection");
-	}
+	CATCH_RETHROW("could not delete collection table")
 }
 
 void MySQLDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi, const CollectionItem& item)
@@ -177,7 +164,7 @@ void MySQLDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi,
 	try
 	{
 		// insert or update?
-		const string& t = lexical_cast<string, ptime>(item.first);
+		const string& t = lexical_cast<string>(item.first);
 		int count;
 		(*m_pSession) << "select count(*) from " << cmi.m_tblname << " where t=?;", use(t), into(count), now;
 
@@ -241,10 +228,7 @@ void MySQLDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi,
 				break;
 		}
 	}
-	catch (...)
-	{
-		throw Xception("could not insert or update collection item");
-	}
+	CATCH_RETHROW("could not insert or update collection item")
 }
 
 void MySQLDatabase::UpdatePurpose (const string& name, const string& purpose) throw (Xception)
@@ -253,10 +237,7 @@ void MySQLDatabase::UpdatePurpose (const string& name, const string& purpose) th
 	{
 		(*m_pSession) << "update TCollections set purpose=? where name=?;", use(purpose), use(name), now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update collection purpose");
-	}
+	CATCH_RETHROW("could not update collection purpose")
 }
 
 #endif // USE_MYSQL
