@@ -14,10 +14,10 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-#ifdef USE_SQLITE
-
 #include "../libpdrx/common.h"
 
+#ifdef USE_SQLITE
+
 #include <Poco/Data/Common.h>
 #include <Poco/Data/SQLite/Connector.h>
 #include <Poco/Data/SQLite/SQLiteException.h>
@@ -31,6 +31,8 @@ using namespace boost::program_options;
 using namespace Poco::Data;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "../libpdrx/config.h"
 #include "db_impl.h"
 
@@ -43,7 +45,7 @@ SQLiteDatabase::SQLiteDatabase (const string& connect, bool verbose)
 void SQLiteDatabase::NoSchemaCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "creating new schema" << endl;
+		encoded::cout << "creating new schema" << endl;
 
 	try
 	{
@@ -82,40 +84,31 @@ void SQLiteDatabase::NoSchemaCallback () throw (Xception)
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not create schema");
-	}
+	CATCH_RETHROW("could not create schema")
 }
 
 void SQLiteDatabase::NoPurposeColumnCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "updating schema" << endl;
+		encoded::cout << "updating schema" << endl;
 
 	try
 	{
 		(*m_pSession) << "alter table TCollections add column purpose varchar(256);", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update schema");
-	}
+	CATCH_RETHROW("could not update schema")
 }
 
 void SQLiteDatabase::NoUnitColumnCallback () throw (Xception)
 {
 	if (m_verbose)
-		cout << "updating schema" << endl;
+		encoded::cout << "updating schema" << endl;
 
 	try
 	{
 		(*m_pSession) << "alter table TCollections add column unit varchar(16);", now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update schema");
-	}
+	CATCH_RETHROW("could not update schema")
 }
 
 void SQLiteDatabase::CreateCollectionInSchema (const string& name, char type, const string& tblname, const string& purpose, const string& unit) throw (Xception)
@@ -142,10 +135,7 @@ void SQLiteDatabase::CreateCollectionInSchema (const string& name, char type, co
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not create collection");
-	}
+	CATCH_RETHROW("could not create collection table")
 }
 
 void SQLiteDatabase::DropCollectionFromSchema (const string& tblname) throw (Xception)
@@ -159,10 +149,7 @@ void SQLiteDatabase::DropCollectionFromSchema (const string& tblname) throw (Xce
 
 		transactor.Commit();
 	}
-	catch (...)
-	{
-		throw Xception("could not delete collection");
-	}
+	CATCH_RETHROW("could not delete collection table")
 }
 
 void SQLiteDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi, const CollectionItem& item)
@@ -170,7 +157,7 @@ void SQLiteDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi
 	try
 	{
 		// insert or update?
-		const string& t = lexical_cast<string, ptime>(item.first);
+		const string& t = lexical_cast<string>(item.first);
 		int count;
 		(*m_pSession) << "select count(*) from " << cmi.m_tblname << " where t=:t;", use(t), into(count), now;
 
@@ -234,10 +221,7 @@ void SQLiteDatabase::InsertOrUpdateCollectionItem (const CollectionMetaInfo& cmi
 				break;
 		}
 	}
-	catch (...)
-	{
-		throw Xception("could not insert or update collection item");
-	}
+	CATCH_RETHROW("could not insert or update collection item")
 }
 
 void SQLiteDatabase::UpdatePurpose (const string& name, const string& purpose) throw (Xception)
@@ -246,10 +230,7 @@ void SQLiteDatabase::UpdatePurpose (const string& name, const string& purpose) t
 	{
 		(*m_pSession) << "update TCollections set purpose=:p where name=:n;", use(purpose), use(name), now;
 	}
-	catch (...)
-	{
-		throw Xception("could not update collection purpose");
-	}
+	CATCH_RETHROW("could not update collection purpose")
 }
 
 #endif // USE_SQLITE
