@@ -22,9 +22,9 @@ using namespace boost::posix_time;
 using namespace boost::gregorian;
 using namespace boost::program_options;
 
-#include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "../libpdrx/config.h"
-#include "../libpdrx/encoding.h"
 #include "db.h"
 #include "in_impl.h"
 
@@ -71,12 +71,7 @@ static const string s_rejected("!!! rejected !!! ");
 
 	static string fix_width (const string& s, int width)
 	{
-		// to format the output for fixed length we need to convert
-		// it from UTF-8 into a character set with a fix character
-		// size, otherwise we get wrong lengths for strings
-		// containing UTF-8-multibyte-characters
-
-		int spaces = width - (ConvertTo(s, GetEncoding("UTF-16")).length() / 2);
+		int spaces = width - lexical_cast<wstring>(s).length();
 		return (spaces <= 0) ? s : s + string(spaces, ' ');
 	}
 
@@ -87,7 +82,7 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 		data_width = 70;	// 10 characters for input on a line
 					// with 80 characters
 
-	cout << "pdr " << VERSION << ", interactive mode (press ? for help)" << endl;
+	encoded::cout << "pdr " << VERSION << ", interactive mode (press ? for help)" << endl;
 
 #ifdef USE_READLINE
 	// disable auto-complete
@@ -128,17 +123,17 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 			if (s_expr.find(s_rejected) == 0)
 				s_expr.erase(0, s_rejected.length());
 		}
-		char* buf = readline(prompt.c_str());
+		char* buf = readline(Encode(prompt, NativeEncoding()).c_str());
 		if (!buf)
 		{
-			cout << endl;
+			encoded::cout << endl;
 			break; // Ctrl-D
 		}
-		line = buf;
+		line = Decode(string(buf), NativeEncoding());
 		free(buf);
 #else
-		cout << prompt;
-		if (!getline(cin, line))
+		encoded::cout << prompt;
+		if (!getline(encoded::cin, line))
 			break;
 #endif
 		trim(line);
@@ -148,7 +143,8 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 
 		if (line == "?" || line == "help")
 		{
-			cout	<< endl
+			encoded::cout
+				<< endl
 				<< "type a command to let pdr do some actions, the following commands are available:" << endl
 				<< endl
 				<< "---line navigation---" << endl
@@ -228,7 +224,8 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 						{
 							if (!line.empty())
 							{
-								cout	<< "deletion of parts of rejections not supported" << endl
+								encoded::cout
+									<< "deletion of parts of rejections not supported" << endl
 									<< "use D to delete the entire rejection or just modify the string" << endl;
 							}
 							else
@@ -277,9 +274,9 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 							}
 						}
 					}
-					catch (Xception& e)
+					catch (const Xception& x)
 					{
-						cerr << e.Message() << endl;
+						encoded::cerr << x.Message(Xception::brief) << endl;
 					}
 					break;
 				}
@@ -295,9 +292,13 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 							const ptime& timestamp = (*I).first;
 							const string expr((*I).second);
 
+							Database::Collections collections;
+							database.GetCollections(collections);
+
 							Database::CollectionsItems items;
-							ptime t = Parse(line, timestamp, false, items);
+							ptime t = Parse(line, timestamp, false, collections, items);
 							database.AddCollectionsItems(items);
+
 							Database::Expressions temp;
 							database.GenerateExpressions(temp, t, "");
 							if (temp.size() == 1)
@@ -321,18 +322,18 @@ void InputInteractive::Do (const Config& config, Database& database) const throw
 							if (expr.find(s_rejected) == 0)
 								database.DeleteRejected(timestamp);
 						}
-						catch (Xception& e)
+						catch (const Xception& x)
 						{
-							cerr << e.Message() << endl;
+							encoded::cerr << x.Message(Xception::brief) << endl;
 						}
 					}
 					break;
 				}
 			}
 		}
-		catch (Xception e)
+		catch (const Xception& x)
 		{
-			cerr << e.Message() << endl;
+			encoded::cerr << x.Message(Xception::brief) << endl;
 		}
 	}
 }
