@@ -27,6 +27,8 @@ using namespace boost::program_options;
 using namespace Poco::Data;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "config_impl.h"
 #include "db_impl.h"
 
@@ -44,17 +46,29 @@ PocoDatabaseImpl::~PocoDatabaseImpl ()
 
 void PocoDatabaseImpl::Connect () throw (Xception)
 {
-	return PocoDatabase::Connect();
+	try
+	{
+		return PocoDatabase::Connect();
+	}
+	CATCH_RETHROW("")
 }
 
 string PocoDatabaseImpl::GetCollectionPurpose (const string& name) const throw (Xception)
 {
-	return PocoDatabase::GetCollectionMetaInfo(name).m_purpose;
+	try
+	{
+		return PocoDatabase::GetCollectionMetaInfo(name).m_purpose;
+	}
+	CATCH_RETHROW("")
 }
 
 string PocoDatabaseImpl::GetCollectionUnit (const string& name) const throw (Xception)
 {
-	return PocoDatabase::GetCollectionMetaInfo(name).m_unit;
+	try
+	{
+		return PocoDatabase::GetCollectionMetaInfo(name).m_unit;
+	}
+	CATCH_RETHROW("")
 }
 
 	inline string selection_key (const string& collection, const ptime& begin, const ptime& end)
@@ -64,22 +78,22 @@ string PocoDatabaseImpl::GetCollectionUnit (const string& name) const throw (Xce
 
 void PocoDatabaseImpl::Select (shared_ptr<Selection>& selection, const string& collection, const ptime& begin, const ptime& end) const throw (Xception)
 {
-	if (m_fast)
-	{
-		Selections::const_iterator I = m_selections.find(selection_key(collection, begin, end));
-		if (I != m_selections.end())
-		{
-			selection = (*I).second; // just return what we know
-			return;
-		}
-	}
-
-	selection->resize(extents[0][2]); // clear any contents
-
-	const CollectionMetaInfo& cmi = GetCollectionMetaInfo(collection);
-
 	try
 	{
+		if (m_fast)
+		{
+			Selections::const_iterator I = m_selections.find(selection_key(collection, begin, end));
+			if (I != m_selections.end())
+			{
+				selection = (*I).second; // just return what we know
+				return;
+			}
+		}
+
+		selection->resize(extents[0][2]); // clear any contents
+
+		const CollectionMetaInfo& cmi = GetCollectionMetaInfo(collection);
+
 		// build the SQL string
 		string sql("select t,");
 		switch (cmi.m_type)
@@ -92,13 +106,13 @@ void PocoDatabaseImpl::Select (shared_ptr<Selection>& selection, const string& c
 		string where;
 		{
 			if (begin != not_a_date_time)
-				where += "t>='" + lexical_cast<string, ptime>(begin) + "'";
+				where += "t>='" + lexical_cast<string>(begin) + "'";
 
 			if (end != not_a_date_time)
 			{
 				if (!where.empty())
 					where += " and ";
-				where += "t<'" + lexical_cast<string, ptime>(end) + "'";
+				where += "t<'" + lexical_cast<string>(end) + "'";
 			}
 		}
 		if (!where.empty())
@@ -139,23 +153,20 @@ void PocoDatabaseImpl::Select (shared_ptr<Selection>& selection, const string& c
 		if (m_fast)
 			m_selections.insert(Selections::value_type(selection_key(collection, begin, end), selection));
 	}
-	catch (...)
-	{
-		throw Xception("error selecting data");
-	}
+	CATCH_RETHROW("error selecting collection data");
 }
 
 void PocoDatabaseImpl::NoSchemaCallback () throw (Xception)
 {
-	throw Xception("no schema found in database, please run pdr to create a schema");
+	THROW("no schema found in database, please run pdr to create a schema");
 }
 
 void PocoDatabaseImpl::NoPurposeColumnCallback () throw (Xception)
 {
-	throw Xception("schema in database is obsolete, please pdr to update the schema");
+	THROW("schema in database is obsolete, please pdr to update the schema");
 }
 
 void PocoDatabaseImpl::NoUnitColumnCallback () throw (Xception)
 {
-	throw Xception("schema in database is obsolete, please run pdr to update the schema");
+	THROW("schema in database is obsolete, please run pdr to update the schema");
 }
