@@ -23,6 +23,8 @@ using namespace boost::gregorian;
 using namespace boost::program_options;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "config_impl.h"
 #include "out_ftree.h"
 #include "diagram.h"
@@ -69,9 +71,13 @@ FDouble::FDouble (double value)
 
 void FDouble::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << format("%g") % m_value;
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << format("%g") % m_value;
+	}
+	CATCH_RETHROW("")
 }
 
 FDateTime::FDateTime (const ptime& value)
@@ -81,9 +87,13 @@ FDateTime::FDateTime (const ptime& value)
 
 void FDateTime::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << lexical_cast<string>(m_value);
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << lexical_cast<string>(m_value);
+	}
+	CATCH_RETHROW("")
 }
 
 FDate::FDate (const date& value)
@@ -93,9 +103,13 @@ FDate::FDate (const date& value)
 
 void FDate::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << lexical_cast<string>(m_value);
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << lexical_cast<string>(m_value);
+	}
+	CATCH_RETHROW("")
 }
 
 FTime::FTime (const time_duration& value)
@@ -105,9 +119,13 @@ FTime::FTime (const time_duration& value)
 
 void FTime::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << lexical_cast<string>(m_value);
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << lexical_cast<string>(m_value);
+	}
+	CATCH_RETHROW("")
 }
 
 FString::FString (const string& value)
@@ -117,9 +135,13 @@ FString::FString (const string& value)
 
 void FString::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << '"' << m_value << '"';
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << '"' << m_value << '"';
+	}
+	CATCH_RETHROW("")
 }
 
 FFormatSpec::FFormatSpec (const fstring& value)
@@ -129,9 +151,13 @@ FFormatSpec::FFormatSpec (const fstring& value)
 
 void FFormatSpec::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << '<' << m_value << '>';
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << '<' << m_value << '>';
+	}
+	CATCH_RETHROW("")
 }
 
 FInt::FInt (int value)
@@ -141,9 +167,13 @@ FInt::FInt (int value)
 
 void FInt::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << m_value;
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << m_value;
+	}
+	CATCH_RETHROW("")
 }
 
 FColor::FColor (size_t value)
@@ -153,9 +183,13 @@ FColor::FColor (size_t value)
 
 void FColor::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << '#' << format("%X") % m_value;
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << '#' << format("%X") % m_value;
+	}
+	CATCH_RETHROW("")
 }
 
 //--- FKeyword -------------------------------------------------------------
@@ -172,9 +206,13 @@ const type_info* FKeyword::Type () const throw (Xception)
 
 void FKeyword::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << m_name;
+	try
+	{
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << m_name;
+	}
+	CATCH_RETHROW("")
 }
 
 any FKeyword::Evaluate (FContext& ) const throw (Xception)
@@ -203,78 +241,90 @@ FExpr::~FExpr ()
 
 const type_info* FExpr::Type () const throw (Xception)
 {
-	if (!m_pFImpl)
+	try
 	{
-		FImpl::ParamTypes paramtypes;
-		foreach (FTree* pParam, m_params)
+		if (!m_pFImpl)
 		{
-			paramtypes.push_back(pParam->Type());
+			FImpl::ParamTypes paramtypes;
+			foreach (FTree* pParam, m_params)
+			{
+				paramtypes.push_back(pParam->Type());
+			}
+			m_pFImpl = fImplFactory.Match(m_func, paramtypes);
 		}
-		m_pFImpl = fImplFactory.Match(m_func, paramtypes);
-	}
 
-	return m_pFImpl->m_returntype;
+		return m_pFImpl->m_returntype;
+	}
+	CATCH_RETHROW("")
 }
 
 void FExpr::Dump (ostream& os, int indent) const throw (Xception)
 {
-	if (indent != -1)
-		os << string(indent * 2, ' ');
-	os << '(' << m_func;
-	if (m_params.empty())
-		os << ')' << endl;
-	else
+	try
 	{
-		bool on_same_line = true;
-		bool all_on_same_line = true;
-		foreach (FTree* pParam, m_params)
-		{
-			if (dynamic_cast<FOneLiner*>(pParam))
-			{
-				if (on_same_line)
-				{
-					os << ' ';
-					pParam->Dump(os, -1);
-				}
-				else
-				{
-					pParam->Dump(os, indent + 1);
-					on_same_line = true;
-				}
-			}
-			else
-			{
-				os << endl;
-				pParam->Dump(os, indent + 1);
-				all_on_same_line = on_same_line = false;
-			}
-		}
-		if (all_on_same_line)
+		if (indent != -1)
+			os << string(indent * 2, ' ');
+		os << '(' << m_func;
+		if (m_params.empty())
 			os << ')' << endl;
 		else
 		{
-			if (on_same_line)
-				os << endl;
-			if (indent != -1)
-				os << string(indent * 2, ' ');
-			os << ')' << endl;
+			bool on_same_line = true;
+			bool all_on_same_line = true;
+			foreach (FTree* pParam, m_params)
+			{
+				if (dynamic_cast<FOneLiner*>(pParam))
+				{
+					if (on_same_line)
+					{
+						os << ' ';
+						pParam->Dump(os, -1);
+					}
+					else
+					{
+						pParam->Dump(os, indent + 1);
+						on_same_line = true;
+					}
+				}
+				else
+				{
+					os << endl;
+					pParam->Dump(os, indent + 1);
+					all_on_same_line = on_same_line = false;
+				}
+			}
+			if (all_on_same_line)
+				os << ')' << endl;
+			else
+			{
+				if (on_same_line)
+					os << endl;
+				if (indent != -1)
+					os << string(indent * 2, ' ');
+				os << ')' << endl;
+			}
 		}
 	}
+	CATCH_RETHROW("")
 }
 
 any FExpr::Evaluate (FContext& context) const throw (Xception)
 {
-	if (!m_pFImpl)
+	try
 	{
-		FImpl::ParamTypes paramtypes;
-		foreach (FTree* pParam, m_params)
+		if (!m_pFImpl)
 		{
-			paramtypes.push_back(pParam->Type());
+			FImpl::ParamTypes paramtypes;
+			foreach (FTree* pParam, m_params)
+			{
+				paramtypes.push_back(pParam->Type());
+			}
+			m_pFImpl = fImplFactory.Match(m_func, paramtypes);
 		}
-		m_pFImpl = fImplFactory.Match(m_func, paramtypes);
-	}
 
-	return m_pFImpl->Evaluate(context, m_params);
+		return m_pFImpl->Evaluate(context, m_params);
+	}
+	CATCH_RETHROW("")
 }
 
 void FExpr::SetFunction (const string& func)
@@ -290,55 +340,63 @@ void FExpr::AddParam (FTree* pFTree)
 //=== helper functions =====================================================
 ostream& operator << (ostream& os, const FTree& fTree) throw (Xception)
 {
-	fTree.Dump(os, 0);
-	return os;
+	try
+	{
+		fTree.Dump(os, 0);
+		return os;
+	}
+	CATCH_RETHROW("")
 }
 
 ostream& operator << (ostream& os, const any& a) throw (Xception)
 {
-	if (a.empty())
-		return os << "nil";
-
-	const type_info& ti = a.type();
-
-	if (ti == typeid(double))
-		return os << format("%g") % any_cast<double>(a);
-
-	if (ti == typeid(Ratio))
-		return os << (format("%g") % any_cast<Ratio>(a).m_numerator) << '/' << (format("%g") % any_cast<Ratio>(a).m_denominator);
-
-	if (ti == typeid(string))
-		return os << any_cast<string>(a);
-
-	if (ti == typeid(ptime))
-		return os << lexical_cast<string>(any_cast<ptime>(a));
-
-	if (ti == typeid(date))
-		return os << lexical_cast<string>(any_cast<date>(a));
-
-	if (ti == typeid(time_duration))
-		return os << lexical_cast<string>(any_cast<time_duration>(a));
-
-	if (ti == typeid(int))
-		return os << any_cast<int>(a);
-
-	if (ti == typeid(shared_ptr<Selection>))
+	try
 	{
-		shared_ptr<Selection> selection = any_cast<shared_ptr<Selection> >(a);
-		const size_t* pDimensions = selection->shape();
-		for (size_t row = 0; row < pDimensions[0]; row++)
-		{
-			if (row > 0)
-				os << endl;
-			for (size_t column = 0; column < pDimensions[1]; column++)
-			{
-				if (column > 0)
-					os << '\t';
-				os << (*selection)[row][column]; // any!
-			}
-		}
-		return os;
-	}
+		if (a.empty())
+			return os << "nil";
 
-	throw Xception("expression returns unknown type");
+		const type_info& ti = a.type();
+
+		if (ti == typeid(double))
+			return os << format("%g") % any_cast<double>(a);
+
+		if (ti == typeid(Ratio))
+			return os << (format("%g") % any_cast<Ratio>(a).m_numerator) << '/' << (format("%g") % any_cast<Ratio>(a).m_denominator);
+
+		if (ti == typeid(string))
+			return os << any_cast<string>(a);
+
+		if (ti == typeid(ptime))
+			return os << lexical_cast<string>(any_cast<ptime>(a));
+
+		if (ti == typeid(date))
+			return os << lexical_cast<string>(any_cast<date>(a));
+
+		if (ti == typeid(time_duration))
+			return os << lexical_cast<string>(any_cast<time_duration>(a));
+
+		if (ti == typeid(int))
+			return os << any_cast<int>(a);
+
+		if (ti == typeid(shared_ptr<Selection>))
+		{
+			shared_ptr<Selection> selection = any_cast<shared_ptr<Selection> >(a);
+			const size_t* pDimensions = selection->shape();
+			for (size_t row = 0; row < pDimensions[0]; row++)
+			{
+				if (row > 0)
+					os << endl;
+				for (size_t column = 0; column < pDimensions[1]; column++)
+				{
+					if (column > 0)
+						os << '\t';
+					os << (*selection)[row][column]; // any!
+				}
+			}
+			return os;
+		}
+
+		THROW("expression returns unknown type");
+	}
+	CATCH_RETHROW("")
 }
