@@ -21,8 +21,11 @@ using namespace boost;
 using namespace boost::posix_time;
 using namespace boost::gregorian;
 using namespace boost::program_options;
+using namespace boost::filesystem;
 
 #include "../libpdrx/datatypes.h"
+#include "../libpdrx/xception.h"
+#include "../libpdrx/conversions.h"
 #include "config_impl.h"
 #include "db.h"
 #include "out_impl.h"
@@ -41,15 +44,24 @@ OutputInteractive::OutputInteractive (const string& option_key)
 
 void OutputInteractive::Do (const Config& config, const Database& database) const throw (Xception)
 {
-	cout << "pdx " << VERSION << ", interactive mode (press q to quit)" << endl;
+	encoded::cout << "pdx " << VERSION << ", interactive mode (press q to quit)" << endl;
 
 #ifdef USE_READLINE
 	// disable auto-complete
 	rl_bind_key('\t', rl_abort);
 
+	// build the history file name depending on the config file name
+	string history_filename;
+	{
+		path p(config.GetConfigFile());
+		if (p.filename() == "pdrxrc")
+			history_filename = (p.parent_path() / "pdx_history").string();
+		else
+			history_filename = (p.parent_path() / ".pdx_history").string();
+	}
+
 	// load the history file if we have one
 	clear_history();
-	const string& history_filename = config.GetConfigFile() + "_history";
 	read_history(history_filename.c_str());
 #endif
 
@@ -57,20 +69,19 @@ void OutputInteractive::Do (const Config& config, const Database& database) cons
 	while (true)
 	{
 #ifdef USE_READLINE
-		char* buf = readline("> ");
+		char* buf = readline(Encode(string("> "), NativeEncoding()).c_str());
 		if (!buf)
 		{
-			cout << endl;
+			encoded::cout << endl;
 			break; // Ctrl-D
 		}
-		line = buf;
+		line = Decode(string(buf), NativeEncoding());
 		free(buf);
 #else
-		cout << "> ";
-		if (!getline(cin, line))
+		encoded::cout << "> ";
+		if (!getline(encoded::cin, line))
 			break;
 #endif
-
 		trim(line);
 		if (line.empty())
 			continue;
@@ -88,7 +99,7 @@ void OutputInteractive::Do (const Config& config, const Database& database) cons
 			{
 				line.erase(0, 1);
 				trim(line);
-				FExpr::fImplFactory.Help(cout, line);
+				FExpr::fImplFactory.Help(encoded::cout, line);
 				continue;
 			}
 
@@ -101,16 +112,16 @@ void OutputInteractive::Do (const Config& config, const Database& database) cons
 
 			auto_ptr<FTree> ft(Parse(line));
 			if (dot)
-				cout << *ft; // dump structure
+				encoded::cout << *ft; // dump structure
 			else
 			{
 				FContext context(config, database);
-				cout << ft->Evaluate(context) << endl;
+				encoded::cout << ft->Evaluate(context) << endl;
 			}
 		}
-		catch (Xception e)
+		catch (const Xception& x)
 		{
-			cerr << e.Message() << endl;
+			encoded::cerr << x.Message(Xception::complete) << endl;
 		}
 	}
 
